@@ -1,4 +1,6 @@
 import os
+import json
+from pathlib import Path
 from urllib.parse import urlparse
 
 from .mixin.admin import AdminMixin
@@ -12,6 +14,26 @@ from .mixin.transaction import TransactionMixin
 from .mixin.version import VersionMixin
 
 
+class ConfigError(Exception):
+    """Error for issues related to the pachyderm config file"""
+    def __init__(self, message):
+        super().__init__(message)
+
+
+class BadClusterDeploymentID(ConfigError):
+    """
+    Error triggered when connected to a cluster that reports back a different
+    cluster deployment ID than what is stored in the config file
+    """
+    def __init__(self, expected_deployment_id, actual_deployment_id):
+        super().__init__("connected to the wrong cluster ('{}' vs '{}')".format(
+            expected_deployment_id,
+            actual_deployment_id
+        ))
+        self.expected_deployment_id = expected_deployment_id
+        self.actual_deployment_id = actual_deployment_id
+
+
 class Client(
     AdminMixin,
     AuthMixin,
@@ -62,6 +84,10 @@ class Client(
         self._auth_token = auth_token
         self._transaction_id = transaction_id
         self._metadata = self._build_metadata()
+        if not auth_token and os.environ.get("PACH_PYTHON_OIDC_TOKEN"):
+            resp = self.authenticate_id_token(os.environ.get("PACH_PYTHON_OIDC_TOKEN"))
+            self._auth_token = resp
+            self._metadata = self._build_metadata()
 
     @classmethod
     def new_in_cluster(cls, auth_token=None, transaction_id=None):
@@ -124,6 +150,77 @@ class Client(
             tls=u.scheme == "grpcs" or u.scheme == "https",
         )
 
+    @classmethod
+    def new_from_config(cls, config_file=None):
+        """
+        Creates a Pachyderm client from a config file, which can either be
+        passed in as a file-like object, or if unset, checks the PACH_CONFIG env
+        var for a path. If that's also unset, it defaults to loading from
+        '~/.pachyderm/config.json'.
+
+        Params:
+
+        * `config_file`: An optional file-like object containing the config
+        json file. If unspecified, we load the config from the default
+        location ('~/.pachyderm/config.json'.)
+        """
+
+        if config_file is not None:
+            j = json.load(config_file)
+        elif "PACH_CONFIG" in os.environ:
+            with open(os.environ.get("PACH_CONFIG"), "r") as config_file:
+                j = json.load(config_file)
+                print("config: {}".format(str(j)))
+        else:
+            try:
+                # Search for config file in default home location
+                with open(str(Path.home() / ".pachyderm/config.json"), "r") as config_file:
+                    j = json.load(config_file)
+            except FileNotFoundError:
+                # If not found, search in "/pachctl" (default mount for spout)
+                with open("/pachctl/config.json", "r") as config_file:
+                    j = json.load(config_file)
+
+        try:
+            active_context = j["v2"]["active_context"]
+        except KeyError:
+            raise ConfigError("no active context")
+
+        try:
+            context = j["v2"]["contexts"][active_context]
+        except KeyError:
+            raise ConfigError("missing active context '{}'".format(active_context))
+
+        auth_token = context.get("session_token")
+        root_certs = context.get("server_cas")
+        transaction_id = context.get("active_transaction")
+
+        pachd_address = context.get("pachd_address")
+        if pachd_address:
+            client = cls.new_from_pachd_address(
+                pachd_address,
+                auth_token=auth_token,
+                root_certs=root_certs,
+                transaction_id=transaction_id
+            )
+        else:
+            port_forwarders = context.get("port_forwarders", {})
+            pachd_port = port_forwarders.get("pachd", 30650)
+            pachd_address = "grpc://localhost:{}".format(pachd_port)
+            client = cls.new_from_pachd_address(
+                pachd_address,
+                auth_token=auth_token,
+                transaction_id=transaction_id
+            )
+
+        expected_deployment_id = context.get("cluster_deployment_id")
+        if expected_deployment_id:
+            cluster_info = client.inspect_cluster()
+            if cluster_info.deployment_id != expected_deployment_id:
+                raise BadClusterDeploymentID(expected_deployment_id, cluster_info.deployment_id)
+
+        return client
+
     @property
     def auth_token(self):
         return self._auth_token
