@@ -3,6 +3,7 @@
 """Tests PFS-related functionality"""
 
 import pytest
+import tempfile
 from io import BytesIO
 
 import python_pachyderm
@@ -225,7 +226,7 @@ def test_put_file_bytes_large():
     client, repo_name = sandbox("put_file_bytes_large")
 
     with client.commit(repo_name) as c:
-        client.put_file_bytes(c, 'file.dat', b'DATA')
+        client.put_file_bytes(c, 'file.dat', b'#'*(21*1024*1024))
 
     commit_infos = list(client.list_commit(repo_name))
     assert len(commit_infos) == 1
@@ -245,6 +246,38 @@ def test_put_file_url():
     assert files[0].file.path == '/index.html'
 
 
+def test_put_file_atomic():
+    client, repo_name = sandbox("put_file_atomic")
+    commit = (repo_name, "master")
+
+    with tempfile.NamedTemporaryFile() as f:
+        with client.put_file_client() as pfc:
+            pfc.put_file_from_fileobj(commit, 'file1.dat', BytesIO(b'DATA1'))
+            pfc.put_file_from_bytes(commit, 'file2.dat', b'DATA1')
+            pfc.put_file_from_url(commit, "index.html", "https://gist.githubusercontent.com/ysimonson/1986773831f6c4c292a7290c5a5d4405/raw/fb2b4d03d317816e36697a6864a9c27645baa6c0/wheel.html")
+
+            f.write(b'DATA3')
+            f.flush()
+            pfc.put_file_from_filepath(commit, "file3.dat", f.name)
+
+    files = list(client.list_file(commit, '.'))
+    assert len(files) == 4
+    assert files[0].file.path == '/file1.dat'
+    assert files[1].file.path == '/file2.dat'
+    assert files[2].file.path == '/file3.dat'
+    assert files[3].file.path == '/index.html'
+
+    # atomic deletes are only supported in 1.11.0 onwards
+    if util.test_pachyderm_version() >= (1, 11, 0):
+        with client.put_file_client() as pfc:
+            pfc.delete_file(commit, "/file1.dat")
+            pfc.delete_file(commit, "/file2.dat")
+            pfc.delete_file(commit, "/file3.dat")
+
+        files = list(client.list_file(commit, '.'))
+        assert len(files) == 1
+        assert files[0].file.path == '/index.html'
+
 def test_copy_file():
     client, repo_name = sandbox("copy_file")
 
@@ -366,10 +399,10 @@ def test_list_file():
     files = list(client.list_file(c, '/'))
     assert len(files) == 2
     assert files[0].size_bytes == 4
-    assert files[0].file_type == python_pachyderm.FileType.FILE
+    assert files[0].file_type == python_pachyderm.FileType.FILE.value
     assert files[0].file.path == "/file1.dat"
     assert files[1].size_bytes == 4
-    assert files[1].file_type == python_pachyderm.FileType.FILE
+    assert files[1].file_type == python_pachyderm.FileType.FILE.value
     assert files[1].file.path == "/file2.dat"
 
 def test_walk_file():
@@ -397,16 +430,16 @@ def test_glob_file():
     files = list(client.glob_file(c, '/*.dat'))
     assert len(files) == 2
     assert files[0].size_bytes == 4
-    assert files[0].file_type == python_pachyderm.FileType.FILE
+    assert files[0].file_type == python_pachyderm.FileType.FILE.value
     assert files[0].file.path == "/file1.dat"
     assert files[1].size_bytes == 4
-    assert files[1].file_type == python_pachyderm.FileType.FILE
+    assert files[1].file_type == python_pachyderm.FileType.FILE.value
     assert files[1].file.path == "/file2.dat"
 
     files = list(client.glob_file(c, '/*1.dat'))
     assert len(files) == 1
     assert files[0].size_bytes == 4
-    assert files[0].file_type == python_pachyderm.FileType.FILE
+    assert files[0].file_type == python_pachyderm.FileType.FILE.value
     assert files[0].file.path == "/file1.dat"
 
 def test_delete_file():
